Imports System.Web.Services
Imports System.Web.Services.Protocols
Imports System.IO
Imports System.Xml


' Create a new custom attribute that will allow
' users to configure the log file. This attribute
' will also tell the ASP.NET system which type to
' load for our extension
<AttributeUsage(AttributeTargets.Method)> Public Class SoapLoggerAttribute
    Inherits SoapExtensionAttribute

#Region "Private Member Variables"
    Private _priority As Integer = 0
    Private _logFile As String = "C:\soap.log"
#End Region

#Region "Contructors"
    Public Sub New()
    End Sub

    Public Sub New(ByVal logFile As String)
        _logFile = logFile
    End Sub
#End Region

#Region "Public Properties"
    Public ReadOnly Property LogFile()
        Get
            LogFile = _logFile
        End Get
    End Property

    Public Overrides ReadOnly Property ExtensionType() As Type
        Get
            ExtensionType = Type.GetType("VBWebSamples.SoapLogger")
        End Get
    End Property

    Public Overrides Property Priority() As Integer
        Get
            Priority = _priority
        End Get
        Set(ByVal Value As Integer)
            _priority = Value
        End Set
    End Property
#End Region

End Class

' Create a new SoapExtension class. When users
' apply the above attribute to a method of their
' web service, this class will be instantiated
' by the ASP.NET system
Public Class SoapLogger
    Inherits SoapExtension

#Region "Private Member Variables"
    Private LogFile As String = ""
    Private SoapStream As Stream
    Private TempStream As Stream
#End Region

#Region "Private Utility Functions"
    Public Sub CopyTextStream(ByVal src As Stream, ByVal dest As Stream)
        Dim reader As New StreamReader(src)
        Dim writer As New StreamWriter(dest)
        writer.WriteLine(reader.ReadToEnd())
        writer.Flush()
    End Sub
#End Region

#Region "Public Overrides"
    Public Overloads Overrides Function GetInitializer(ByVal serviceType As Type) As Object
        ' we need to read some service configuration from the attribute
        ' so differ creating an initializer
        GetInitializer = Nothing
    End Function
    Public Overloads Overrides Function GetInitializer(ByVal methodInfo As LogicalMethodInfo, ByVal attribute As SoapExtensionAttribute) As Object
        ' whatever we return here will be passed back to
        ' our initialize method every time the ASP.NET system
        ' needs to create a new instance of this extension class
        Dim loggerAttr As SoapLoggerAttribute = attribute
        GetInitializer = loggerAttr.LogFile
    End Function
    Public Overrides Sub Initialize(ByVal initializer As Object)
        ' grab the logfile name that we returned in
        ' GetInitializer
        LogFile = initializer
    End Sub
    Public Overrides Function ChainStream(ByVal stream As Stream) As Stream
        ' by overriding ChainStream we can
        ' cause the ASP.NET system to use
        ' our stream for buffering SOAP messages
        ' rather than the default stream.
        ' we will store off the original stream
        ' so we can pass the data back down to the ASP.NET system
        ' in original stream that it created.
        SoapStream = stream
        TempStream = New MemoryStream()
        ChainStream = TempStream
    End Function
    Public Overrides Sub ProcessMessage(ByVal message As SoapMessage)
        ' this method will be called several times during
        ' the processing of a SOAP request. The ASP.NET system
        ' tells us which stage the SOAP request is at with the
        ' Stage property of the SoapMessage class
        Select Case message.Stage
            Case SoapMessageStage.BeforeDeserialize
                ' copy the SOAP request from the network stream
                ' into our memory buffer
                CopyTextStream(SoapStream, TempStream)
                Dim fs As New FileStream(LogFile, FileMode.Append, FileAccess.Write)
                Dim sw As New StreamWriter(fs)
                sw.WriteLine("** BEGIN SOAP REQUEST: {0}", DateTime.Now)
                sw.Flush()

                ' copy the mem buffer stream to the log file
                TempStream.Position = 0
                CopyTextStream(TempStream, fs)
                sw.WriteLine("** END SOAP REQUEST")
                sw.Flush()
                fs.Close()

                ' reset the memory buffer position
                ' so the ASP.NET system can parse and
                ' decode the message
                TempStream.Position = 0
            Case SoapMessageStage.AfterSerialize
                Dim fs As New FileStream(LogFile, FileMode.Append, FileAccess.Write)
                Dim sw As New StreamWriter(fs)
                sw.WriteLine("** BEGIN SOAP RESPONSE: {0}", DateTime.Now)
                sw.Flush()
                TempStream.Position = 0
                CopyTextStream(TempStream, fs)
                sw.WriteLine("** END SOAP RESPONSE")
                sw.Flush()

                ' copy the memory beffered response
                ' to the network stream
                TempStream.Position = 0
                CopyTextStream(TempStream, SoapStream)

                fs.Close()
        End Select
    End Sub
#End Region

End Class


Public Class Book
    Public Title As String
    Public Publisher As String
End Class

Public Class Author
    Public FirstName As String
    Public LastName As String
    Public Email As String
    Public Books As Book()
End Class

Public Class SoapExtensionSample
    Inherits System.Web.Services.WebService

#Region " Web Services Designer Generated Code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Web Services Designer.
        InitializeComponent()

        'Add your own initialization code after the InitializeComponent() call

    End Sub

    'Required by the Web Services Designer
    Private components As System.ComponentModel.Container

    'NOTE: The following procedure is required by the Web Services Designer
    'It can be modified using the Web Services Designer.  
    'Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThroughAttribute()> Private Sub InitializeComponent()
        components = New System.ComponentModel.Container()
    End Sub

    Overloads Overrides Sub Dispose()
        'CODEGEN: This procedure is required by the Web Services Designer
        'Do not modify it using the code editor.
    End Sub

#End Region

#Region "Web Methods"

    <WebMethod(), SoapLogger("C:\vbsoap.log")> Public Function GetAuthorInfo() As Author
        Dim info As New Author()
        info.FirstName = "Bob"
        info.LastName = "Tabor"
        info.Email = "bob@technicallead.com"
        ReDim info.Books(2)
        info.Books(0) = New Book()
        info.Books(0).Title = "Microsoft .NET WebServices"
        info.Books(0).Publisher = "SAMS Publishing"
        info.Books(1) = New Book()
        info.Books(1).Title = "Microsoft .NET WebServices 2nd Edition"
        info.Books(1).Publisher = "SAMS Publishing"

        GetAuthorInfo = info
    End Function

#End Region

End Class
